/* Shared library add-on to iptables to add state tracking support. */
#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include "packet-filter/userspace/iptables.h"
#include "packet-filter/extentions/ipt_state.h"

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"state v%s options:\n"
" --state [!] [INVALID|ESTABLISHED|NEW|RELATED]\n"
"				State to [not] match\n"
"\n", NETFILTER_VERSION);
}

static struct option opts[] = {
	{ "state", 1, 0, '1' },
	{0}
};

static void
init(union ipt_matchinfo *matchinfo, unsigned int *nfcache)
{
	/* Can't cache this. */
	*nfcache |= NFC_UNKNOWN;
}

static void
parse_state(const char *state, struct ipt_state_info *sinfo)
{
	if (strcasecmp(state, "NEW") == 0)
		sinfo->state = IPT_STATE_NEW;
	else if (strcasecmp(state, "ESTABLISHED") == 0)
		sinfo->state = IPT_STATE_ESTABLISHED;
	else if (strcasecmp(state, "RELATED") == 0)
		sinfo->state = IPT_STATE_RELATED;
	else if (strcasecmp(state, "INVALID") == 0)
		sinfo->state = IPT_STATE_INVALID;
	else
		exit_error(PARAMETER_PROBLEM, "Bad state `%s'", state);
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      struct ipt_entry *entry)
{
	struct ipt_state_info *sinfo
		= (struct ipt_state_info *)&entry->matchinfo.other;
	switch (c) {
	case '1':
		if (check_inverse(optarg, &invert))
			optind++;
		parse_state(argv[optind-1], mac);
		if (invert)
			sinfo->invert = 1;
		*flags = 1;
		break;

	default:
		return 0;
	}

	return 1;
}

/* Final check; must have specified --mac. */
static void final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM, "You must specify `--state'");
}

/* Prints out the union ipt_matchinfo. */
static void print(const union ipt_matchinfo *info, int numeric)
{
	struct ipt_state_info *sinfo
		= (struct ipt_state_info *)&info.other;

	printf("state %s", sinfo->invert ? "!" : "");
	switch (sinfo->state) {
	case IPT_STATE_INVALID:
		printf("INVALID");
		break;
	case IPT_STATE_NEW:
		printf("NEW");
		break;
	case IPT_STATE_RELATED:
		printf("RELATED");
		break;
	case IPT_STATE_ESTABLISHED:
		printf("ESTABLISHED");
		break;
	case default:
		printf("*ERROR* (%u)", sinfo->state);
	}
}

struct iptables_match state
= { NULL,
    "state",
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    opts
};

void _init(void)
{
	register_match(&state);
}
