/* Simple kernel module to drop packets marked invalid */
#include <linux/config.h>

#include <linux/module.h>
#include <linux/netfilter_ipv4.h>
#include "conntrack/ip_conntrack.h"

static int hook = -1;
MODULE_PARM(hook, "i");

/* The work comes in here from netfilter.c. */
static unsigned int
invalid_hook(unsigned int hook,
	     struct sk_buff **pskb,
	     const struct net_device *indev,
	     const struct net_device *outdev)
{
	struct ip_conntrack_tuple_hash *h;
	enum ip_conntrack_info ctinfo;
	unsigned long flags;

	LOCK_IRQSAVE(&ip_conntrack_lck, flags);
	ctinfo = ip_conntrack_findinfo(*pskb, &h);
	UNLOCK_IRQRESTORE(&ip_conntrack_lck, flags);

	if (ctinfo == IP_CT_INVALID)
		return NF_DROP;
	else return NF_ACCEPT;
}

static struct nf_hook_ops dropinvalid_ops
= { { NULL, NULL }, invalid_hook, NULL, PF_INET, 0, 0 };

int init_module(void)
{
	if (hook < 0 || hook >= NF_IP_NUMHOOKS) {
		printk("Must specify hook=[0-%i]\n", NF_IP_NUMHOOKS-1);
		return -EINVAL;
	}

	dropinvalid_ops.hooknum = hook;

	/* Register hook */
	return nf_register_hook(&dropinvalid_ops);
}

void
cleanup_module(void)
{
	nf_unregister_hook(&dropinvalid_ops);
}
