#! /bin/sh

# Loopback and locally-generated packets have various of their own
# problems.  We use netcat here to generate locally-produced packets.

# Need job control.
set -m

# First no conntracks
if [ "`cat /proc/net/ip_conntrack`" != "" ]
then
    echo Non-empty /proc/net/ip_conntrack:
    cat /proc/net/ip_conntrack
    exit 1
fi

# Map loopback to an external interface.
../NAT/userspace/ipnatctl -I -d 127.0.0.1 -b dest -t $TAP0NET.2 || exit 1

./rcv_ip 2 1 < /dev/tap0 > $TMPFILE &
sleep 1

echo test | nc -w 1 -u -p 100 127.0.0.1 101

if wait %./rcv_ip; then :;
else
    echo rcv_ip failed after local mapping.
    exit 1
fi

if [ "`cat $TMPFILE`" != "$TAP0 $TAP0NET.2 33 17 100 101 " ]
then
    echo rcv_ip after local mapping unexpected:
    echo \`"`cat $TMPFILE`"\'
    exit 1
fi

if [ `wc -l < /proc/net/ip_conntrack` != 1 ]
then
    echo Unexpected /proc/net/ip_conntrack lines after local UDP:
    cat /proc/net/ip_conntrack
    exit 1
fi

if grep "^udp      17 [0-9]* src=127.0.0.1 dst=127.0.0.1 sport=100 dport=101 \[UNREPLIED\] src=$TAP0NET.2 dst=$TAP0 sport=101 dport=100 $" /proc/net/ip_conntrack > /dev/null
then :
else
    echo Unexpected /proc/net/ip_conntrack after local UDP:
    echo \`"`cat /proc/net/ip_conntrack`"\'
    exit 1
fi

# Reply should not cause new conntrack.
./gen_ip $TAP0NET.2 $TAP0 100 17 101 100 > /dev/tap0 || exit 1

if [ `wc -l < /proc/net/ip_conntrack` != 1 ]
then
    echo Unexpected /proc/net/ip_conntrack lines after UDP reply:
    cat /proc/net/ip_conntrack
    exit 1
fi

if grep "^udp      17 [0-9]* src=127.0.0.1 dst=127.0.0.1 sport=100 dport=101 src=$TAP0NET.2 dst=$TAP0 sport=101 dport=100 $" /proc/net/ip_conntrack > /dev/null
then :
else
    echo Unexpected /proc/net/ip_conntrack after UDP reply:
    cat /proc/net/ip_conntrack
    exit 1
fi

exit 0
